from __future__ import print_function
import readline
import json
import re
from .config.config import read_from_user
from intent_schema import IntentSchema
from argparse import ArgumentParser


def print_description(intent):
    print ("<> Enter data for <{intent}> OR Press enter with empty string to move onto next intent"
           .format(intent=intent["intent"]))
    print ("<> Enter '<' to delete last training utterance")
    print ("<> Sample utterance to remind you of the format:")
    print (">> what is the recipe for {ravioli|Food} ?")
    if len(intent["slots"]) > 0:
        print ("<> Available slots for this intent")
        for slot in intent["slots"]:
            print (" - - ", slot["name"], "<TYPE: {}>".format(slot["type"])) 

            
def validate_input_format(utterance, intent):
    """ TODO add handling for bad input"""
    slots = {slot["name"] for slot in intent["slots"]}
    split_utt = re.split("{(.*)}", utterance)
    banned = set("-/\\()^%$#@~`-_=+><;:")
    for token in split_utt:
        if (banned & set(token)):
            print (" - Banned character found in substring", token)
            print (" - Banned character list", banned)
            return False

        if "|" in token:
            split_token = token.split("|")
            if len(split_token)!=2:
                print (" - Error, token is incorrect in", token, split_token)
                return False

            word, slot = split_token
            if slot.strip() not in slots:
                print (" -", slot, "is not a valid slot for this Intent, valid slots are", slots)
                return False
    return True


def lowercase_utterance(utterance):
    split_utt = re.split("({.*})", utterance)
    def lower_case_split(token):
        if "|" in token:
            phrase, slot = token.split("|")
            return "|".join([phrase.strip().lower(), slot.strip()])
        else:
            return token.lower()
    return " ".join([lower_case_split(token) for token in split_utt])
    
        
def generate_training_data(schema):
    print ("Loaded intent schema, populating intents")
    training_data = []
    for intent in schema.get_intents():
        print_description(intent)
        keep_prompting = True
        while keep_prompting:            
            utterance = read_from_user(str,
                                       str(len(training_data))+". "+intent["intent"]+'\t')
            if utterance.strip() == "":
                keep_prompting = False
            elif utterance.strip() == "<":
                print (" - Discarded utterance: ", training_data.pop())
            elif validate_input_format(utterance, intent):
                training_data.append("\t".join([intent["intent"], lowercase_utterance(utterance)]))
            else:
                print (" - Discarded utterance:", utterance)
    return training_data                


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--intent_schema', '-i', required=True)
    parser.add_argument('--output', '-o', default='utterances.txt')
    args = parser.parse_args()
    intent_schema = IntentSchema.from_filename(args.intent_schema)    
    with open(args.output, 'w') as utterance_file:
        utterance_file.write("\n".join(generate_training_data(intent_schema)))



'''
Abstractions around IntentSchema class. 
'''
from __future__ import print_function
import json
from collections import OrderedDict
from argparse import ArgumentParser
import os
from .config.config import read_from_user, load_builtin_slots


class IntentSchema(object):
    '''
    Wrapper class to manipulate Intent Schema
    '''
    def __init__(self, json_obj=None):
        if json_obj:
            # Use existing intent schema 
            self._obj = json_obj
        else:
            # Create one from scratch
            self._obj = OrderedDict({ "intents" : [] })

            # These intents are basically always needed
            # for certification 
            self.add_intent('AMAZON.HelpIntent')
            self.add_intent('AMAZON.StopIntent')
            self.add_intent('AMAZON.CancelIntent')
            
    def add_intent(self, intent_name, slots=None):
        if not slots: slots = []
        intent = OrderedDict()
        intent ['intent'], intent['slots'] = intent_name, slots        
        self._obj['intents'].append(intent)
        
        
    def build_slot(self, slot_name, slot_type):
        slot = OrderedDict()
        slot['name'], slot['type'] = slot_name, slot_type
        return slot
    
    def __str__(self):
        return json.dumps(self._obj, indent=2)

    
    def get_intents(self):
        return self._obj['intents']     

    def get_intent_names(self):
        return [intent['intent'] for intent in self.get_intents()]
    
    @classmethod
    def interactive_build(self, fpath=None):
        intent_schema = IntentSchema.from_filename(fpath)
        print ("How many intents would you like to add")
        num = read_from_user(int)
        for i in range(num):
            intent_schema._add_intent_interactive(intent_num=i+1)
        return intent_schema

    def save_to_file(self, filename):
        with open(filename, 'w') as fp:
            print(self, file=fp)
            
    def _add_intent_interactive(self, intent_num=0):        
        '''
        Interactively add a new intent to the intent schema object 
        '''
        print ("Name of intent number : ", intent_num)
        slot_type_mappings = load_builtin_slots()
        intent_name = read_from_user(str)
        print ("How many slots?")        
        num_slots = read_from_user(int)
        slot_list = []
        for i in range(num_slots):
            print ("Slot name no.", i+1)
            slot_name = read_from_user(str).strip()
            print ("Slot type? Enter a number for AMAZON supported types below,"
                   "else enter a string for a Custom Slot")
            print (json.dumps(slot_type_mappings, indent=True))
            slot_type_str = read_from_user(str)
            try: slot_type = slot_type_mappings[int(slot_type_str)]['name'] 
            except: slot_type = slot_type_str
            slot_list += [self.build_slot(slot_name, slot_type)]                    
        self.add_intent(intent_name, slot_list)                        

    
    @classmethod
    def from_filename(self, filename):
        '''
        Build an IntentSchema from a file path 
        creates a new intent schema if the file does not exist, throws an error if the file
        exists but cannot be loaded as a JSON
        '''
        if os.path.exists(filename):
            with open(filename) as fp:
                return IntentSchema(json.load(fp, object_pairs_hook=OrderedDict))
        else:
            print ('File does not exist')
            return IntentSchema()

def from_filename(fname):
    return IntentSchema.from_filename(fname)

        
if __name__ == '__main__':

    parser = ArgumentParser()
    parser.add_argument('--intent_schema', '-i', required=True) 
    parser.add_argument('--overwrite', '-o', action='store_true',
                        default=False)
    args = parser.parse_args()

    if not args.overwrite:
        print ('In APPEND mode')
        intent_schema = IntentSchema.interactive_build(args.intent_schema)
    else:
        print ('In OVERWRITE mode')
        intent_schema = IntentSchema.interactive_build()        

    print ("Write to file:", args.intent_schema,"? (y/n)")
    dec = read_in(str).strip().lower()
    if dec == "y":
        intent_schema.save_to_file(args.intent_schema)
    elif dec == "n":
        pass



from . import alexa_io

''' 
Setting up some nice abstractions around good object oritented code.
'''

ResponseBuilder = alexa_io.ResponseBuilder
alexa = alexa_io.VoiceHandler()
Request = alexa_io.Request



"""
This is the basic config file, encapsulating all configuration options
ALL FILES SHOULD LOAD THEIR CONFIGURATIONS FROM THIS CENTRAL LOCATION
"""
from __future__ import print_function
import os
import json

# ---- Helper Functions ----

# Get path relative to the current file 
path_relative_to_file = lambda rel_path: os.path.normpath(os.path.join(os.path.dirname(__file__), rel_path))

# Load a json file as an object
load_json_schema = lambda schema_location : json.load(open(schema_location))


def read_from_user(input_type, *args, **kwargs):
    '''
    Helper function to prompt user for input of a specific type 
    e.g. float, str, int 
    Designed to work with both python 2 and 3 
    Yes I know this is ugly.
    '''

    def _read_in(*args, **kwargs):
        while True:
            try: tmp =  raw_input(*args, **kwargs)
            except NameError: tmp =  input(*args, **kwargs)
            try: return input_type(tmp)
            except: print ('Expected type', input_type)

    return _read_in(*args, **kwargs)

# Location of AMAZON.BUILTIN slot types
BUILTIN_SLOTS_LOCATION = path_relative_to_file(os.path.join('..', 'data', 'amazon_builtin_slots.tsv'))

def load_builtin_slots():
    '''
    Helper function to load builtin slots from the data location
    '''
    builtin_slots = {}
    for index, line in enumerate(open(BUILTIN_SLOTS_LOCATION)):
        o =  line.strip().split('\t')
        builtin_slots[index] = {'name' : o[0],
                                'description' : o[1] } 
    return builtin_slots




from distutils.core import setup

setup(name='ask-alexa-pykit',
      version='0.5.1',
      description="Minimalist SDK for developing skills for Amazon's Alexa Skills Kit",
      author='Anjishnu Kumar',
      author_email='anjishnu.kr@gmail.com',
      url='https://github.com/anjishnu/ask-alexa-pykit',
      packages=['ask', 'ask.config'], 
      package_data={'ask.config': ['../data/*']},
      license='MIT',
)
